/*********
*
* In the name of the Father, and of the Son, and of the Holy Spirit.
*
* This file is part of BibleTime's source code, http://www.bibletime.info/
*
* Copyright 1999-2020 by the BibleTime developers.
* The BibleTime source code is licensed under the GNU General Public License
* version 2.0.
*
**********/

#ifndef BTSTRINGJOIN_H
#define BTSTRINGJOIN_H

#define BT_STRING_JOIN_(a, b) a ## b
#define BT_STRING_JOIN(a, b) BT_STRING_JOIN_(a, b)

#endif /* BTSTRINGJOIN_H */
